@@ -305,7 +305,7 @@ struct TaskAccumulate
 	  found = true;
 	}
       
-      if (! learn_finished) {
+      if (! learn_finished && mapper_.empty()) {
 	found = true;
 	
 	gradient_.clear();
@@ -442,7 +442,7 @@ void learn_online(const Learner& learner,
   ostream_ptr_set_type ostreams(mpi_size);
   istream_ptr_set_type istreams(mpi_size);
   
-  queue_type mapper;
+  queue_type mapper; // one for gradient, another for terminatin
   queue_type reducer;
   
   task_type task(learner,
@@ -472,6 +472,7 @@ void learn_online(const Learner& learner,
     boost::thread worker(boost::ref(task));
     
     bool finished = false;
+    bool full = false;
     
     int non_found_iter = 0;
     for (;;) {
@@ -479,16 +480,17 @@ void learn_online(const Learner& learner,
       
       // reduce samples...
       for (int rank = 0; rank != mpi_size; ++ rank)
-	if (rank != mpi_rank && istreams[rank] && istreams[rank]->test()) {
-	  if (istreams[rank]->read(buffer))
-	    reducer.push_swap(buffer);
-	  else
-	    istreams[rank].reset();
-	  
-	  buffer.clear();
-	  found = true;
-	}
-
+	if (rank != mpi_rank)
+	  while (istreams[rank] && istreams[rank]->test()) {
+	    if (istreams[rank]->read(buffer))
+	      reducer.push_swap(buffer);
+	    else
+	      istreams[rank].reset();
+	    
+	    buffer.clear();
+	    found = true;
+	  }
+      
       // check termination...
       if (! finished && std::count(istreams.begin(), istreams.end(), istream_ptr_type()) == mpi_size) {
 	reducer.push(buffer_type());
@@ -497,7 +499,7 @@ void learn_online(const Learner& learner,
       
       // bcast...
       // first, get the encoded buffer from mapper
-      if (mapper.pop_swap(buffer, true)) {
+      if (! full && mapper.pop_swap(buffer, true)) {
 	buffer_ptr_type buffer_ptr;
 	
 	if (! buffer.empty()) {
@@ -514,22 +516,28 @@ void learn_online(const Learner& learner,
       }
       
       // second, bcast...
+      full = false;
+      
       for (int rank = 0; rank != mpi_size; ++ rank)
-	if (rank != mpi_rank && ostreams[rank] && ostreams[rank]->test() && ! buffers[rank].empty()) {
-	  if (! buffers[rank].front()) {
-	    // termination!
-	    if (! ostreams[rank]->terminated())
-	      ostreams[rank]->terminate();
-	    else {
-	      ostreams[rank].reset();
+	if (rank != mpi_rank) {
+	  while (ostreams[rank] && ostreams[rank]->test() && ! buffers[rank].empty()) {
+	    if (! buffers[rank].front()) {
+	      // termination!
+	      if (! ostreams[rank]->terminated())
+		ostreams[rank]->terminate();
+	      else {
+		ostreams[rank].reset();
+		buffers[rank].erase(buffers[rank].begin());
+	      }
+	    } else {
+	      ostreams[rank]->write(*(buffers[rank].front()));
 	      buffers[rank].erase(buffers[rank].begin());
 	    }
-	  } else {
-	    ostreams[rank]->write(*(buffers[rank].front()));
-	    buffers[rank].erase(buffers[rank].begin());
+	    
+	    found = true;
 	  }
 	  
-	  found = true;
+	  full |= (buffers[rank].size() > 1024);
 	}
       
       // termination condition
@@ -566,7 +574,8 @@ void learn_online(const Learner& learner,
     
     if (debug && mpi_rank == 0)
       std::cerr << "log-likelihood: " << static_cast<double>(log_likelihood) << std::endl
-		<< "perplexity: " << std::exp(- static_cast<double>(log_likelihood)) << std::endl;
+		<< "entropy: "    << std::exp(- static_cast<double>(log_likelihood)) << std::endl
+		<< "perplexity: " << (- static_cast<double>(log_likelihood) / std::log(2.0)) << std::endl;
     
     if (debug && mpi_rank == 0)
       std::cerr << "cpu time:    " << end.cpu_time() - start.cpu_time() << std::endl
