@@ -14,9 +14,6 @@ import re
 import subprocess
 import cStringIO
 
-### for find_executable!
-import distutils.spawn
-
 from optparse import OptionParser, make_option
 
 opt_parser = OptionParser(
@@ -88,13 +85,32 @@ opt_parser = OptionParser(
     
     make_option("--pbs", default=None, action="store_true",
                 help="PBS for launching processes"),
-    make_option("--pbs-queue", default="ltg", action="store", type="string",
+    make_option("--pbs-queue", default="", action="store", type="string",
                 help="PBS queue for launching processes (default: %default)", metavar="NAME"),
 
     ## debug messages
     make_option("--debug", default=0, action="store", type="int"),
     ])
 
+def find_executable(executable, paths=[]):
+    ### taken from distutils.spawn
+    
+    paths += os.environ['PATH'].split(os.pathsep)
+    
+    base, ext = os.path.splitext(executable)
+
+    if (sys.platform.startswith('win') or sys.platform.startswith('os2')) and (ext != '.exe'):
+        executable = executable + '.exe'
+
+    if not os.path.isfile(executable):
+        for p in paths:
+            f = os.path.join(p, executable)
+            if os.path.isfile(f):
+                # the file exists, we have a shot at spawn working
+                return f
+        return None
+    else:
+        return executable
 
 def run_command(command):
     try:
@@ -159,15 +175,14 @@ class Program:
 class PBS:
     def __init__(self, queue=""):
         self.queue = queue
-        self.qsub = 'qsub'
+        self.qsub = find_executable('qsub')
         
-        # how to find binary location...?
-        if not distutils.spawn.find_executable('qsub'):
+        if not self.qsub:
             raise ValueError, "no qsub in your executable path?"
 
     def run(self, command="", name="name", memory=0.0, mpi=None, threads=1, logfile=None):
 
-        popen = subprocess.Popen(['qsub', '-S', '/bin/sh'], stdin=subprocess.PIPE)
+        popen = subprocess.Popen([self.qsub, '-S', '/bin/sh'], stdin=subprocess.PIPE)
         pipe = popen.stdin
         
         pipe.write("#!/bin/sh\n")
@@ -249,19 +264,20 @@ class MPI:
             self.hosts_file = os.path.realpath(hosts_file)
 
         self.bindir = self.dir
-	
-        for binprog in ['mpirun']:
-            if self.bindir:
-                prog = os.path.join(self.bindir, 'bin', binprog)
-                if not os.path.exists(prog) or os.path.isdir(prog):
-                    prog = os.path.join(self.bindir, binprog)
-                    if not os.path.exists(prog) or os.path.isdir(prog):
-                        raise ValueError, prog + " does not exist at " + self.bindir
-                    
-                setattr(self, binprog, prog)
-            else:
-                setattr(self, binprog, binprog)
-                
+
+        paths = []
+        if self.bindir:
+            paths = [os.path.join(self.bindir, 'bin'), self.bindir]
+        
+        binprog = find_executable('openmpirun', paths)
+        if not binprog:
+            binprog = find_executable('mpirun', paths)
+
+        if not binprog:
+            raise ValueError, "no openmpirun nor mpirun?"
+
+        setattr(self, 'mpirun', binprog)
+        
     def run(self, command, logfile=None):
         mpirun = self.mpirun
         if self.number > 0:
@@ -353,19 +369,12 @@ class CICADA:
                         'cicada_eval',
                         'cicada_filter_config',
                         'cicada_filter_weights',):
-	    
-	    for bindir in bindirs:
-		prog = os.path.join(bindir, binprog)
+            
+            prog = find_executable(binprog, bindirs)
+            if not prog:
+                raise ValueError, binprog + ' does not exist'
                 
-                if not os.path.exists(prog): continue
-                if os.path.isdir(prog): continue
-                
-                setattr(self, binprog, prog)
-                break
-
-	    if not hasattr(self, binprog):
-		raise ValueError, binprog + ' does not exist'
-
+            setattr(self, binprog, prog)
 
 def learn_algorithms(command):
     pattern = re.compile(r"\s+--learn-(\S+)\s*")
