@@ -535,7 +535,7 @@ namespace utils
       double logprob = parameter.log_likelihood();
       
       if (! customers) return logprob;
-      
+
       if (discount > 0.0) {
 	if (strength == 0.0)
 	  logprob += tables * std::log(discount) + utils::mathop::lgamma(tables) - utils::mathop::lgamma(customers);
@@ -547,19 +547,21 @@ namespace utils
 	const double lg = utils::mathop::lgamma(1.0 - discount);
 	
 	typename dish_set_type::const_iterator diter_end = dishes.end();
-	for (typename dish_set_type::const_iterator diter = dishes.begin(); diter != diter_end; ++ diter) {
-	  const location_type& loc = *diter;
-	  
-	  typename location_type::const_iterator titer_end = loc.end();
-	  for (typename location_type::const_iterator titer = loc.begin(); titer != titer_end; ++ titer)
-	    logprob += (utils::mathop::lgamma(titer->first - discount) - lg) * titer->second;
-	}
+	for (typename dish_set_type::const_iterator diter = dishes.begin(); diter != diter_end; ++ diter) 
+	  if (! diter->empty()) {
+	    const location_type& loc = *diter;
+	    
+	    typename location_type::const_iterator titer_end = loc.end();
+	    for (typename location_type::const_iterator titer = loc.begin(); titer != titer_end; ++ titer)
+	      logprob += (utils::mathop::lgamma(titer->first - discount) - lg) * titer->second;
+	  }
       } else if (discount == 0.0) {
 	logprob += utils::mathop::lgamma(strength) + tables * std::log(strength) - utils::mathop::lgamma(strength + tables);
 	
 	typename dish_set_type::const_iterator diter_end = dishes.end();
 	for (typename dish_set_type::const_iterator diter = dishes.begin(); diter != diter_end; ++ diter)
-	  logprob += utils::mathop::lgamma(diter->size_table());
+	  if (! diter->empty())
+	    logprob += utils::mathop::lgamma(diter->size_table());
       } else
 	throw std::runtime_error("negative discount?");
       
@@ -741,7 +743,7 @@ namespace utils
 					     0.0,
 					     num_iterations,
 					     32 * num_iterations);
-	
+
 	if (has_discount_prior()) 
 	  parameter.discount = slice_sampler(discount_sampler,
 					     parameter.discount,
